import os
import json
import glob
import argparse

def plot(labels, values):
  import matplotlib.pyplot as plt
  import numpy as np
    
  # plt.style.use('seaborn-white')
  plt.style.use('ggplot')
  
  x_pos = [i for i, _ in enumerate(labels)]
  width = 0.35

  fig, ax = plt.subplots(figsize=(8,8))
  
  ax.bar(x_pos, values, width, color='green')
  ax.set_xlabel('SSD Model', fontsize=14)
  ax.set_ylabel('Max Throughput (KiB/s)', fontsize=14)
  ax.set_title("Max Throughput of Chameleon SSDs", fontsize=14)

  ax.set_xticks(x_pos, labels, fontsize=8)

  # ax.xticks(x_pos, labels, rotation=30)
  # plt.xticks(rotation=90)

  fig.savefig('plot.png')
  
#   x = np.arange(len(labels))

#   fig, ax = plt.subplots()
#   rects1 = ax.bar(x, values, width, label='Max Throughput')

#   ax.set_ylabel('Max Throughput')
#   ax.set_title('Max Throughput of Chameleon SSDs')
#   ax.set_xticks(x, labels)
#   ax.legend()

#   ax.bar_label(rects1, padding=3)

#   fig.tight_layout()
#   fig.savefig('plot.png')

def main():
  parser = argparse.ArgumentParser(prog='fio-parser.py', description='%(prog)s is used to parse fio output files')
  parser.add_argument('--directory','-d',type=str,help='Specify the directory with fio output files to parse.  If none if provided, ./ is used')
  args = parser.parse_args()
  # jsonfiles = glob.glob("{}/*/**.json".format(args.directory))
    
  # print(args.directory)

  global_max_throughput = 0  
  result={}
  # for i in range(len(jsonfiles)):
  for d in os.listdir(args.directory):
    if '.ipynb' in d:
        continue
    fio_result = json.load(open(f'{args.directory}/{d}/{d}-fio.json', 'r'))
    # fio_ssds = json.load(open(f'{args.directory}/{d}/{d}-ssd.json', 'r'))
    
    jobs = fio_result['jobs']
    print(d)
    
    max_throughput = 0
    for job in jobs:
      model, _ = job['jobname'].split('_')
      bw_max = job['read']['bw_max']
      print(f'  job = {job["jobname"]}, read (KiB) = {job["read"]["io_kbytes"]}, IOPS = {job["read"]["iops"]}, max throughput = {bw_max} KiB/s')                     
      max_throughput = bw_max if bw_max > max_throughput else max_throughput
      global_max_throughput = bw_max if bw_max > global_max_throughput else global_max_throughput
      if model in result:
            result[model] = bw_max if bw_max > result[model] else result[model]
      else:
            result[model] = bw_max

    print(f'node-{d}, max throughput = {max_throughput} KiB/s')
    print()
    
  
  print(result)
  print()

  # plot_data = open('plot.dat', 'w')
  print('Models:')
  count = 0
  labels = []
  values = []
  for model in result:
        # plot_data.write(f'{count} "{model}" {result[model]}\n')
        print(f'- {model} = {result[model]} KiB/s')
        labels.append(model)
        values.append(result[model])
        count += 1
  print()
  print(f'global max throughput = {global_max_throughput} KiB/s')
    
  plot(labels, values)

if __name__ == '__main__':
    main()