#!/usr/bin/env bash
set -u

NODE_RESERVATION_IDS=()
NODE_IP_IDS=()
NODE_IP_ADDRS=()

NODE_TYPE="${NODE_TYPE:-compute_skylake}"

public_network_id=$(openstack network show public -f value -c id)

echo "Requesting lease for:"
echo " - $NUM_NODES bare metal node (type(s)=$NODE_TYPES)"
echo " - $NUM_NODES Floating IPs"
echo

echo "Reserving $NUM_NODES node(s)"

for (( i=0; i<${NUM_NODES}; i++ ));
do
    NODE_TYPE=${NODE_TYPES[$i]}
    lease_name="$RESOURCE_NAME-$NODE_TYPE"

    echo "Reserve lease $lease_name with type $NODE_TYPE..."
    blazar lease-create \
      --physical-reservation min=1,max=1,resource_properties="[\"=\", \"\$node_type\", \"$NODE_TYPE\"]" \
      --reservation resource_type=virtual:floatingip,network_id="$public_network_id",amount=1 \
      --start-date "$(date -u +'%Y-%m-%d %H:%M' -d'+1 minute')" \
      --end-date "$(date -u +'%Y-%m-%d %H:%M' -d'+1 day')" \
      "$lease_name"

    echo "Waiting for lease $lease_name to start"
    echo
    attempts=0
    wait_lease() {
      sleep 5
      attempts=$((attempts+1))
      printf "."
      [[ $attempts -gt 60 ]] && return 1
    }
    until [[ $(blazar lease-show "$lease_name" -f value -c status) == "ACTIVE" ]]; do wait_lease; done
    echo
    echo "Lease started successfully!"
    echo
    
    reservations=$(blazar lease-show "$lease_name" -f json \
      | jq -r '.reservations')
    host_reservation_id=$(jq -rs 'map(select(.resource_type=="physical:host"))[].id' <<<"$reservations")
    fip_reservation_id=$(jq -rs 'map(select(.resource_type=="virtual:floatingip"))[].id' <<<"$reservations")

    fip=$(openstack floating ip list --tags "reservation:$fip_reservation_id" -f json)
    fip_id=$(jq -r '.[0]["ID"]' <<<"$fip")
    fip_addr=$(jq -r '.[0]["Floating IP Address"]' <<<"$fip")
    
    NODE_RESERVATION_IDS+=($host_reservation_id)
    NODE_IP_IDS+=($fip_id)
    NODE_IP_ADDRS+=($fip_addr)
done

export FLOATING_IP_IDS=("${NODE_IP_IDS[@]}")
export FLOATING_IPS=("${NODE_IP_ADDRS[@]}")
export RESERVATION_IDS=("${NODE_RESERVATION_IDS[@]}")
echo
echo "Stack parameters:"
echo " - floating_ips (exported as FLOATING_IP_IDS and FLOATING_IPS):"
for (( i=0; i<${NUM_NODES}; i++ ));
do
    echo "       - ${FLOATING_IP_IDS[$i]} (addr: ${FLOATING_IPS[$i]})"
done
echo " - reservation_ids (exported as RESERVATION_IDS):"
for value in "${RESERVATION_IDS[@]}"
do
    echo "       - $value"
done